#!/bin/bash

PROJECT_ROOT=$(git rev-parse --show-toplevel)
LICENSE_LOCATION="$PROJECT_ROOT"/scripts/LICENSE_HEADER
NUMLINES=$(wc -l "$LICENSE_LOCATION" | cut -d\  -f1)
LICENSE=$(sed "s/{DATE_Y}/$(date +"%Y")/" "$LICENSE_LOCATION")
VERSIONED_GO_FILES=$(git ls-tree --full-tree --name-only -r HEAD | grep "\.go$")
EXCLUDE=(
    "Code generated by"
    "David Lazar"
    "Go Authors"
    "Google Inc"
    "Prometheus Authors"
)
FILTER=$(IFS="|" ; echo "${EXCLUDE[*]}")
INPLACE=false
VERBOSE=false
RETURN_VALUE=0

usage() {
    echo "check_license"
    echo
    echo "Usage: $0 [args]"
    echo
    echo "Args:"
    echo "-i    Edit in-place."
    echo "-v    Verbose, same as doing \`head -n 15\` on each file."
    echo
}

while [ "$1" != "" ]; do
    case "$1" in
        -i)
            INPLACE=true
            ;;
        -v) VERBOSE=true
            ;;
        -h)
            usage
            exit 0
            ;;
        *)
            echo "$(tput setaf 1)[ERROR]$(tput sgr0) Unknown option $1"
            usage
            exit 1
            ;;
    esac
    shift
done

for FILE in $VERSIONED_GO_FILES
do
    # https://en.wikipedia.org/wiki/Cat_(Unix)#Useless_use_of_cat
    if [[ $LICENSE != $(<"$PROJECT_ROOT/$FILE" head -n "$NUMLINES") ]]
    then
        if <"$PROJECT_ROOT/$FILE" head -n "$NUMLINES" | tr "\n" " " | grep -qvE "$FILTER"
        then
            RETURN_VALUE=1

            if ! $VERBOSE
            then
                if $INPLACE
                then
                    cat <(echo "$LICENSE") "$PROJECT_ROOT/$FILE" > "$PROJECT_ROOT/$FILE".1 &&
                        mv "$PROJECT_ROOT/$FILE"{.1,}
                else
                    echo "$FILE"
                fi
            else
                echo -e "\n$(tput setaf 2)$FILE$(tput sgr0)"
                <"$PROJECT_ROOT/$FILE" head -n "$NUMLINES"
                echo
            fi
        fi
    fi
done

exit $RETURN_VALUE

